@@ -18,6 +18,8 @@
   2005-2007
 */
 
+/* put this first so we get <config.h> and <gettext.h> immediately: */
+#include "tuxmath.h"
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -32,7 +34,6 @@
 #include "game.h"
 #include "fileops.h"
 #include "setup.h"
-#include "tuxmath.h"
 #include "mathcards.h"
 #include "titlescreen.h"
 #include "options.h"  
@@ -75,6 +76,7 @@ typedef struct comet_type {
   int bonus;
   int zapped;
   MC_FlashCard flashcard;
+  Uint32 time_started;
 } comet_type;
 
 /* Local (to game.c) 'globals': */
@@ -156,7 +158,7 @@ static int check_exit_conditions(void);
 static void draw_numbers(const char* str, int x, int y);
 static void game_set_message(game_message *,char *,int x, int y);
 static void game_clear_message(game_message*);
-static void game_write_message(const game_message*);
+static void game_write_message(const game_message* msg);
 static void game_write_messages(void);
 static void draw_led_console(void);
 static void draw_question_counter(void);
@@ -658,7 +660,7 @@ void game_handle_help(void)
   frame = 0;
 
   // Write the introductory text
-  game_set_message(&s1,"Welcome to TuxMath!",-1,50);
+  game_set_message(&s1,_("Welcome to TuxMath!"),-1,50);
 
 #ifndef NOSOUND
   if (Opts_UsingSound())
@@ -675,9 +677,9 @@ void game_handle_help(void)
   if (quit_help)
     return;
 
-  game_set_message(&s2,"Your mission is to save your", left_edge, 100);
-  game_set_message(&s3,"penguins' igloos from the", left_edge, 135);
-  game_set_message(&s4,"falling comets.", left_edge, 170);
+  game_set_message(&s2,_("Your mission is to save your"), left_edge, 100);
+  game_set_message(&s3,_("penguins' igloos from the"), left_edge, 135);
+  game_set_message(&s4,_("falling comets."), left_edge, 170);
 
   frame_start = frame;
   while (frame-frame_start < 5*FPS && !(quit_help = help_renderframe_exit()));  // wait 5 more secs
@@ -696,10 +698,10 @@ void game_handle_help(void)
     return;
 
   if (comets[0].alive == 1) {
-    game_set_message(&s1,"Stop a comet by typing",left_edge,100);
-    game_set_message(&s2,"the answer to the math problem",left_edge,135);
-    game_set_message(&s3,"and hitting 'space' or 'enter'.",left_edge,170);
-    game_set_message(&s4,"Try it now!",left_edge,225);
+    game_set_message(&s1,_("Stop a comet by typing"),left_edge,100);
+    game_set_message(&s2,_("the answer to the math problem"),left_edge,135);
+    game_set_message(&s3,_("and hitting 'space' or 'enter'."),left_edge,170);
+    game_set_message(&s4,_("Try it now!"),left_edge,225);
 
     speed = 0;
     while (comets[0].alive && !(quit_help = help_renderframe_exit()));
@@ -707,7 +709,7 @@ void game_handle_help(void)
       return;
   }
 
-  game_set_message(&s1,"Good shot!",left_edge,100);
+  game_set_message(&s1,_("Good shot!"),left_edge,100);
   game_clear_message(&s2);
   game_clear_message(&s3);
   game_clear_message(&s4);
@@ -717,11 +719,11 @@ void game_handle_help(void)
   while (frame-frame_start < 3*FPS && !(quit_help = help_renderframe_exit()));  // wait 3 secs
   
   speed = 2;
-  game_set_message(&s1,"If an igloo gets hit by a comet,",left_edge,100);
-  game_set_message(&s2,"it melts. But don't worry, the",left_edge,135);
-  game_set_message(&s3,"penguin is OK!",left_edge,170);
-  game_set_message(&s4,"Just watch what happens:",left_edge,225);
-  game_set_message(&s5,"(Press a key to start)",left_edge,260);
+  game_set_message(&s1,_("If an igloo gets hit by a comet,"),left_edge,100);
+  game_set_message(&s2,_("it melts. But don't worry: the"),left_edge,135);
+  game_set_message(&s3,_("penguin is OK!"),left_edge,170);
+  game_set_message(&s4,_("Just watch what happens:"),left_edge,225);
+  game_set_message(&s5,_("(Press a key to start)"),left_edge,260);
 
   key_pressed = 0;
   while (!key_pressed && !(quit_help = help_renderframe_exit()));
@@ -734,19 +736,19 @@ void game_handle_help(void)
   while (!(comets[0].expl) && !(quit_help = help_renderframe_exit()));  // wait 3 secs
   if (quit_help)
     return;
-  game_set_message(&s4,"Notice the answer",left_edge,comets[0].y-100);
+  game_set_message(&s4,_("Notice the answer"),left_edge,comets[0].y-100);
   help_renderframe_exit();
   SDL_Delay(4000);
+  game_clear_message(&s4);
 
   frame_start = frame;
   while (frame-frame_start < 5*FPS && !(quit_help = help_renderframe_exit()));  // wait 5 secs
   if (quit_help)
     return;
 
-  game_set_message(&s1,"If it gets hit again, the",left_edge,100);
-  game_set_message(&s2,"penguin leaves.",left_edge,135);
-  game_set_message(&s3, "(Press a key when ready)",left_edge,200);
-  game_clear_message(&s4);
+  game_set_message(&s1,_("If it gets hit again, the"),left_edge,100);
+  game_set_message(&s2,_("penguin leaves."),left_edge,135);
+  game_set_message(&s3,_("(Press a key when ready)"),left_edge,200);
 
   key_pressed = 0;
   while (!key_pressed && !(quit_help = help_renderframe_exit()));
@@ -765,8 +767,8 @@ void game_handle_help(void)
     return;
   
   help_controls.laser_enabled = 1;
-  game_set_message(&s1,"You can fix the igloos",left_edge,100);
-  game_set_message(&s2,"by stopping bonus comets.",left_edge,135);
+  game_set_message(&s1,_("You can fix the igloos"),left_edge,100);
+  game_set_message(&s2,_("by stopping bonus comets."),left_edge,135);
   help_add_comet(2,MC_OPER_ADD,2,4);
   comets[0].bonus = 1;
   frame_start = frame;
@@ -775,11 +777,11 @@ void game_handle_help(void)
     return;
   if (comets[0].alive)
     speed = 0;
-  game_set_message(&s3,"Zap it now!",left_edge,225);
+  game_set_message(&s3,_("Zap it now!"),left_edge,225);
   while (comets[0].alive && !(quit_help = help_renderframe_exit()));
   if (quit_help)
     return;
-  game_set_message(&s1,"Great job!",left_edge,100);
+  game_set_message(&s1,_("Great job!"),left_edge,100);
   game_clear_message(&s2);
   game_clear_message(&s3);
   frame_start = frame;
@@ -793,10 +795,10 @@ void game_handle_help(void)
     return;
 
 
-  game_set_message(&s1,"Quit at any time by pressing",left_edge,100);
-  game_set_message(&s2,"'Esc' or clicking the 'X'",left_edge,135);
-  game_set_message(&s3,"in the upper right corner.",left_edge,170);
-  game_set_message(&s4,"Do it now, and then play!",left_edge,225);
+  game_set_message(&s1,_("Quit at any time by pressing"),left_edge,100);
+  game_set_message(&s2,_("'Esc' or clicking the 'X'"),left_edge,135);
+  game_set_message(&s3,_("in the upper right corner."),left_edge,170);
+  game_set_message(&s4,_("Do it now, and then play!"),left_edge,225);
 
   help_controls.x_is_blinking = 1;
   while (!help_renderframe_exit());
@@ -1022,6 +1024,7 @@ void game_handle_demo(void)
 void game_handle_answer(void)
 {
   int i, num, lowest, lowest_y;
+  Uint32 ctime;
 
   if (!doing_answer)
   {
@@ -1061,6 +1064,14 @@ void game_handle_answer(void)
   {
     MC_AnsweredCorrectly(&(comets[lowest].flashcard));
 
+    /* Store the time the question was present on screen (do this */
+    /* in a way that avoids storing it if the time wrapped around */
+    ctime = SDL_GetTicks();
+    if (ctime > comets[lowest].time_started) {
+      MC_AddTimeToList((float)(ctime - comets[lowest].time_started)/1000);
+    }
+    
+
     /* Destroy comet: */
     comets[lowest].expl = COMET_EXPL_START;
     comets[lowest].zapped = 1;
@@ -1192,8 +1203,10 @@ void game_handle_comets(void)
   /* Handle comets. Since the comets also are the things that trigger
      changes in the cities, we set some flags in them, too. */
   int i, this_city;
+  Uint32 ctime;
+
   num_comets_alive = 0;
-      
+
   /* Clear the threatened flag on each city */
   for (i = 0; i < NUM_CITIES; i++)
     cities[i].threatened = 0;
@@ -1228,7 +1241,14 @@ void game_handle_comets(void)
         /* Tell MathCards about it - question not answered correctly: */
         MC_NotAnsweredCorrectly(&(comets[i].flashcard));
 
-        /* Record data for feedback */
+	/* Store the time the question was present on screen (do this */
+	/* in a way that avoids storing it if the time wrapped around */
+	ctime = SDL_GetTicks();
+	if (ctime > comets[i].time_started) {
+	  MC_AddTimeToList((float)(ctime - comets[i].time_started)/1000);
+	}
+
+        /* Record data for speed feedback */
 	/* Do this only for cities that are alive; dead cities */
         /* might not get much protection from the player */
 	if (Opts_UseFeedback() && cities[this_city].hits_left) {
@@ -1318,7 +1338,7 @@ void game_handle_comets(void)
       {
         if (add_comet())
         {
-	  num_attackers--;
+          num_attackers--;
         }
       }
     }
@@ -1329,8 +1349,8 @@ void game_handle_comets(void)
 	if (!check_extra_life()) {
 	  /* Time for the next wave! */
 	  wave++;
-	  reset_level();
-	}
+          reset_level();
+        }
       }
     }
   }
@@ -1610,8 +1630,8 @@ int check_extra_life(void)
 
 void game_handle_extra_life(void)
 {
-  int i,igloo_top,num_below,status,direction;
-  Uint8 cloud_transparency;
+  // This handles the animation sequence during the rebuilding of an igloo
+  int i,igloo_top,num_below_igloo,direction;
 
   if (cloud.status == EXTRA_LIFE_ON) {
 
@@ -1622,37 +1642,32 @@ void game_handle_extra_life(void)
      }
 #endif
 
+    // Get the cloud moving in the right direction, if not yet "parked"
     direction = 2*(cloud.city < NUM_CITIES/2) - 1;
     if (direction*cloud.x < direction*cities[cloud.city].x) {
       cloud.x += direction*PENGUIN_WALK_SPEED;
-      SDL_SetAlpha(images[IMG_CLOUD],SDL_SRCALPHA | SDL_RLEACCEL,
-		   SDL_ALPHA_OPAQUE);
     }
     else {
+      // Cloud is "parked," handle the snowfall and igloo rebuilding
       cities[cloud.city].status = CITY_REBUILDING;
-      for (i = 0, num_below = 0; i < NUM_SNOWFLAKES; i++) {
-	cloud.snowflake_y[i] += SNOWFLAKE_SPEED;
-	if (cloud.snowflake_y[i] > cloud.y)
-	  num_below++;
-      }
-      cloud_transparency = (Uint8) ((float) (NUM_SNOWFLAKES-num_below)/NUM_SNOWFLAKES
-	* (float) SDL_ALPHA_OPAQUE);
-      status = SDL_SetAlpha(images[IMG_CLOUD],SDL_SRCALPHA,cloud_transparency);
       igloo_top = screen->h - igloo_vertical_offset
 	- images[IMG_IGLOO_INTACT]->h;
+      for (i = 0, num_below_igloo = 0; i < NUM_SNOWFLAKES; i++) {
+	cloud.snowflake_y[i] += SNOWFLAKE_SPEED;
+	if (cloud.snowflake_y[i] > igloo_top)
+	  num_below_igloo++;
+      }
       if (cloud.snowflake_y[NUM_SNOWFLAKES-1] > igloo_top) {
 	cities[cloud.city].hits_left = 2;
-	cities[cloud.city].img = IMG_IGLOO_INTACT;
-      }
-      else if (cloud.snowflake_y[2*NUM_SNOWFLAKES/3] > igloo_top &&
-	       cities[cloud.city].hits_left == 0)
-	cities[cloud.city].img = IMG_IGLOO_REBUILDING2;
-      else if (cloud.snowflake_y[NUM_SNOWFLAKES/3] > igloo_top &&
-	       cities[cloud.city].hits_left == 0)
-	cities[cloud.city].img = IMG_IGLOO_REBUILDING1;
-      if (cloud.snowflake_y[NUM_SNOWFLAKES/3] > igloo_top) {
+	cities[cloud.city].img = IMG_IGLOO_INTACT; // completely rebuilt
+      } else if (cities[cloud.city].hits_left == 0) {
+	// We're going to draw one of the blended igloos
+	// FIXME: It's a hack to encode a blended igloo with a negative number!
 	penguins[cloud.city].layer = 0;
 	cities[cloud.city].layer = 1;
+	if (num_below_igloo < 3)
+	  num_below_igloo = 0;   // Don't show progress until a few have fallen
+	cities[cloud.city].img = -((float) (num_below_igloo)/NUM_SNOWFLAKES) * NUM_BLENDED_IGLOOS;
       }
       if (cloud.snowflake_y[NUM_SNOWFLAKES-1] > screen->h - igloo_vertical_offset) {
 	/* exit rebuilding when last snowflake at igloo bottom */
@@ -1790,7 +1805,13 @@ void game_draw(void)
 	  max_layer = steam[i].layer;
 	if (cities[i].layer == current_layer &&
 	    cities[i].img != IMG_CITY_NONE) {
-	  this_image = images[cities[i].img];
+	  // Handle the blended igloo images, which are encoded
+	  // (FIXME) with a negative image number
+	  if (cities[i].img <= 0)
+	    this_image = blended_igloos[-cities[i].img];
+	  else
+	    this_image = images[cities[i].img];
+	  //this_image = blended_igloos[frame % NUM_BLENDED_IGLOOS];
 	  dest.x = cities[i].x - (this_image->w / 2);
 	  dest.y = (screen->h) - (this_image->h) - igloo_vertical_offset;
 	  if (cities[i].img == IMG_IGLOO_MELTED3 ||
@@ -2414,6 +2435,9 @@ int add_comet(void)
   printf ("\nadd_comet(): formula string is: %s", comets[found].flashcard.formula_string);
   #endif
 
+  /* Record the time at which this comet was created */
+  comets[found].time_started = SDL_GetTicks();
+
   /* comet slot found and question found so return successfully: */
   return 1;
 }
