@@ -33,6 +33,11 @@ int questions_pending = 0;
 int unanswered = 0;
 int starting_length = 0;
 
+/* For keeping track of timing data */
+float* time_per_question_list = NULL;
+int length_time_per_question_list = 0;
+int length_alloc_time_per_question_list = 0;
+
 /* "private" function prototypes:                        */
 /*                                                       */
 /* these are for internal use by MathCards only - like   */
@@ -48,9 +53,11 @@ static MC_MathQuestion* insert_node(MC_MathQuestion* first, MC_MathQuestion* cur
 static MC_MathQuestion* append_node(MC_MathQuestion* list, MC_MathQuestion* new_node);
 static MC_MathQuestion* remove_node(MC_MathQuestion* first, MC_MathQuestion* n);
 static MC_MathQuestion* delete_list(MC_MathQuestion* list);
+static int copy_node(MC_MathQuestion* original, MC_MathQuestion* copy);
 static int list_length(MC_MathQuestion* list);
+static int randomize_list(MC_MathQuestion** list);
 
-static MC_MathQuestion* randomize_list(MC_MathQuestion* list);
+int comp_randomizer(const void *a, const void *b);
 static MC_MathQuestion* pick_random(int length, MC_MathQuestion* list);
 static int compare_node(MC_MathQuestion* first, MC_MathQuestion* other);
 static int already_in_list(MC_MathQuestion* list, MC_MathQuestion* ptr);
@@ -58,8 +65,10 @@ static int int_to_bool(int i);
 static int sane_value(int i);
 static int abs_value(int i);
 static int randomly_keep(void);
+static int floatCompare(const void *v1,const void *v2);
 
 static void print_list(FILE* fp,MC_MathQuestion* list);
+void print_vect_list(FILE* fp, MC_MathQuestion** vect, int length);
 static void print_node(FILE* fp, MC_MathQuestion* ptr);
 
 /* these functions are dead code unless compiling with debug turned on: */
@@ -67,7 +76,7 @@ static void print_node(FILE* fp, MC_MathQuestion* ptr);
 static void print_card(MC_FlashCard card);
 static void print_counters(void);
 static MC_MathQuestion* create_node_copy(MC_MathQuestion* other);
-static int copy_node(MC_MathQuestion* original, MC_MathQuestion* copy);
+
 static MC_FlashCard*    create_card_from_node(MC_MathQuestion* node);
 #endif
 
@@ -222,6 +231,14 @@ int MC_StartGame(void)
   delete_list(wrong_quests);
   wrong_quests = NULL;
 
+  /* clear the time list */
+  if (time_per_question_list != NULL) {
+    free(time_per_question_list);
+    time_per_question_list = NULL;
+    length_time_per_question_list = 0;
+    length_alloc_time_per_question_list = 0;
+  }
+
   /* set up new list with pointer to top: */
   question_list = generate_list();
 
@@ -290,7 +307,12 @@ int MC_StartGameUsingWrongs(void)
 
     /* initialize lists for new game: */
     delete_list(question_list);
-    question_list = randomize_list(wrong_quests);
+    if(!randomize_list(&wrong_quests)) {
+      fprintf(stderr, "Error during randomization of wrong_quests!\n");
+      /* Punt on trying wrong question list, just run normal game */
+      return MC_StartGame();
+    }
+    question_list = wrong_quests;
     wrong_quests = 0;
     next_wrong_quest = 0; 
    /* initialize counters for new game: */
@@ -573,6 +595,41 @@ int MC_ListQuestionsLeft(void)
 }
 
 
+/*  Store the amount of time a given flashcard was      */
+/*  visible on the screen. Returns 1 if the request     */
+/*  succeeds, 0 otherwise.                              */
+int MC_AddTimeToList(float t)
+{
+  int newsize = 0;
+  float *newlist;
+
+  /* This list will be allocated in an STL-like manner: when the       */
+  /* list gets full, allocate an additional amount of storage equal    */
+  /* to the current size of the list, so that only O(logN) allocations */
+  /* will ever be needed. We therefore have to keep track of 2 sizes:  */
+  /* the allocated size, and the actual number of items currently on   */
+  /* the list.                                                         */
+  if (length_time_per_question_list >= length_alloc_time_per_question_list) {
+    /* The list is full, allocate more space */
+    newsize = 2*length_time_per_question_list;
+    if (newsize == 0)
+      newsize = 100;
+    newlist = realloc(time_per_question_list,newsize*sizeof(float));
+    if (newlist == NULL) {
+      #ifdef MC_DEBUG
+      printf("\nError: allocation for time_per_question_list failed\n");
+      #endif
+      return 0;
+    }
+    time_per_question_list = newlist;
+    length_alloc_time_per_question_list = newsize;
+  }
+
+  /* Append the time to the list */
+  time_per_question_list[length_time_per_question_list++] = t;
+  return 1;
+}
+
 /* Frees heap memory used in program:                   */
 void MC_EndGame(void)
 {
@@ -587,6 +644,11 @@ void MC_EndGame(void)
     math_opts = 0;
   }
 
+  free(time_per_question_list);
+  time_per_question_list = NULL;
+  length_alloc_time_per_question_list = 0;
+  length_time_per_question_list = 0;
+
   initialized = 0;
 }
 
@@ -1977,6 +2039,16 @@ int MC_NumNotAnsweredCorrectly(void)
 }
 
 
+/* Report the median time per question */
+float MC_MedianTimePerQuestion(void)
+{
+  if (length_time_per_question_list == 0)
+    return 0;
+
+  qsort(time_per_question_list,length_time_per_question_list,sizeof(float),floatCompare);
+  return time_per_question_list[length_time_per_question_list/2];
+}
+
 /* Implementation of "private methods" - (cannot be called from outside
 of this file) */
 
@@ -2385,7 +2457,11 @@ MC_MathQuestion* generate_list(void)
   /*  now shuffle list if desired: */
   if (math_opts->randomize)
   {
-    top_of_list = randomize_list(top_of_list); 
+    if(!randomize_list(&top_of_list))
+    { 
+      fprintf(stderr, "Error during list randomization!\n");
+      return NULL;
+    }
   }
 
   #ifdef MC_DEBUG
@@ -2571,27 +2647,40 @@ MC_FlashCard* create_card_from_node(MC_MathQuestion* node)
 }
 #endif
 
-#ifdef MC_DEBUG
+
 /* FIXME take care of strings */
 /* this one copies the contents, including pointers; both nodes must be allocated */
 int copy_node(MC_MathQuestion* original, MC_MathQuestion* copy)
 {
-  if (!original || !copy)
+  if (!original)
   {
-    printf("\nIn copy_node(): invalid pointer as argument.\n");
-    fprintf(stderr, "\nIn copy_node(): invalid pointer as argument.\n");
+    fprintf(stderr, "\nIn copy_node(): invalid 'original' pointer arg.\n");
     return 0;
   }  
+  if (!copy)
+  {
+    fprintf(stderr, "\nIn copy_node(): invalid 'copy' pointer arg.\n");
+    return 0;
+  }  
+
   copy->card.num1 = original->card.num1;
   copy->card.num2 = original->card.num2;
   copy->card.num3 = original->card.num3;
   copy->card.operation = original->card.operation;
   copy->card.format = original->card.format;
+  strncpy(copy->card.formula_string,
+          original->card.formula_string,
+          MC_FORMULA_LEN);
+  strncpy(copy->card.answer_string,
+          original->card.answer_string,
+          MC_ANSWER_LEN);
   copy->next = original->next;
   copy->previous = original->previous;
+  copy->randomizer = original->randomizer;
   return 1;
 }
-#endif
+
+
 
 
 /* this puts the node into the list AFTER the node pointed to by current */
@@ -2693,16 +2782,32 @@ void print_list(FILE* fp, MC_MathQuestion* list)
     fprintf(fp, "\nprint_list(): list empty or pointer invalid\n");
     return;
   }
- {
-  MC_MathQuestion* ptr = list;
-  while (ptr)
+
   {
-    print_node(fp, ptr);
-    ptr = ptr->next;
+    MC_MathQuestion* ptr = list;
+    while (ptr)
+    {
+      print_node(fp, ptr);
+      ptr = ptr->next;
+    }
   }
 }
-}
 
+void print_vect_list(FILE* fp, MC_MathQuestion** vect, int length)
+{
+  if (!vect)
+  {
+    fprintf(fp, "\nprint_vect_list(): list empty or pointer invalid\n");
+    return;
+  }
+
+  {
+    int i = 0;
+    for(i = 0; i < length; i++) 
+      print_node(fp, vect[i]);
+  }
+  fprintf(stderr, "Leaving print_vect_list()\n");
+}
 
 /* Don't need this much now that formula_string part of card struct:  */
 void print_node(FILE* fp, MC_MathQuestion* ptr)
@@ -2712,7 +2817,8 @@ void print_node(FILE* fp, MC_MathQuestion* ptr)
     return;
   }
 
-  fprintf(fp, "\n%s", ptr->card.formula_string);
+  fprintf(fp, "%s\n", ptr->card.formula_string);
+  /*fprintf(fp, "randomizer = %d\n", ptr->randomizer);*/
 }  
 
 
@@ -2760,64 +2866,82 @@ int list_length(MC_MathQuestion* list)
 
 
 
-MC_MathQuestion* randomize_list(MC_MathQuestion* old_list)
+
+
+/* This is a new implementation written in an attempt to avoid       */
+/* the O(n^2) performance problems seen with the old randomization   */
+/* function. The list is created as a vector, but is for now still   */
+/* made a linked list to minimize changes needed elsewhere.          */
+/* The argument is a pointer to the top of the old list.  This extra */
+/* level of indirection allows the list to be shuffled "in-place".   */
+/* The function returns 1 if successful, 0 on errors.                */
+
+static int randomize_list(MC_MathQuestion** old_list)
 {
+  MC_MathQuestion* old_tmp = *old_list;
+  MC_MathQuestion** tmp_vect = NULL;
 
-  MC_MathQuestion* old_tmp = 0;
-  MC_MathQuestion* new_list = 0;
-  MC_MathQuestion* new_tmp =0;
+  int i = 0;
+  int old_length = list_length(old_tmp);
 
-  int old_length = list_length(old_list);
-  int new_length = 0;
-
-  #ifdef MC_DEBUG
-  printf("\nEntering randomize_list()");
-  printf("\nBefore randomization:");
-  printf("\nPrinting old_list:");
-  print_list(stdout, old_list);
-  printf("\nPrinting new_list:");
-  print_list(stdout, new_list);
-  #endif
+  /* set random seed: */
+  srand(time(0));  
 
 
-  while (old_length && old_list)
+  /* Allocate vector and set ptrs to nodes in old list: */
+
+  /* Allocate a list of pointers, not space for the nodes themselves: */
+  tmp_vect = (MC_MathQuestion**)malloc(sizeof(MC_MathQuestion*) * old_length);
+  /* Set each pointer in the vector to the corresponding node: */
+  for (i = 0; i < old_length; i++)
   {
-    old_tmp = pick_random(old_length, old_list);
-    new_tmp = pick_random(new_length, new_list);
-
-    if (old_tmp)
-    {
-      old_list = remove_node(old_list, old_tmp);
-      new_list = insert_node(new_list, new_tmp, old_tmp);
-      old_length--;
-      new_length++;
-    }
-    else
-    {
-      #ifdef MC_DEBUG
-      printf("\nUnexpected exit!");
-      printf("\nAfter randomization:");
-      printf("\nPrinting old_list:");
-      print_list(stdout, old_list);
-      printf("\nPrinting new_list:");
-      print_list(stdout, new_list);
-      printf("\nLeaving randomize_list()");
-      #endif
-
-      return new_list;
-    }
+    tmp_vect[i] = old_tmp;
+    tmp_vect[i]->randomizer = rand();
+    old_tmp = old_tmp->next;
   }
 
-  #ifdef MC_DEBUG
-  printf("\nAfter randomization:");
-  printf("\nPrinting old_list:");
-  print_list(stdout, old_list);
-  printf("\nPrinting new_list:");
-  print_list(stdout, new_list);
-  printf("\nLeaving randomize_list()");
-  #endif
+  /* Now simply sort on 'tmp_vect[i]->randomizer' to shuffle list: */
+  qsort(tmp_vect, old_length,
+        sizeof(MC_MathQuestion*),
+        comp_randomizer);
 
-  return new_list;
+  /* Re-create pointers to provide linked-list functionality:      */
+  /* (stop at 'old_length-1' because we dereference tmp_vect[i+1]) */
+  for(i = 0; i < old_length - 1; i++)
+  {
+    if (!tmp_vect[i])
+    {
+      fprintf(stderr, "Invalid pointer!\n");
+      return 0;
+    }
+    tmp_vect[i]->next = tmp_vect[i+1];
+    tmp_vect[i+1]->previous = tmp_vect[i];
+  }
+  /* Handle end cases: */
+  tmp_vect[0]->previous = NULL;
+  tmp_vect[old_length-1]->next = NULL;
+
+  /* Now arrange for arg pointer to indirectly point to first element! */
+  *old_list = tmp_vect[0];
+  free(tmp_vect);
+  return 1;
+}
+
+
+
+/* This is needed for qsort(): */
+int comp_randomizer (const void* a, const void* b)
+{
+
+  int int1 = (*(const struct MC_MathQuestion **) a)->randomizer;
+  int int2 = (*(const struct MC_MathQuestion **) b)->randomizer;
+
+  if (int1 > int2)
+    return 1;
+  else if (int1 == int2)
+    return 0;
+  else
+    return -1;
 }
 
 MC_MathQuestion* pick_random(int length, MC_MathQuestion* list)
@@ -2933,3 +3057,19 @@ int randomly_keep(void)
   else
     return 0;
 }
+
+/* Compares two floats (needed for sorting in MC_MedianTimePerQuestion) */
+int floatCompare(const void *v1,const void *v2)
+{
+  float f1,f2;
+
+  f1 = *((float *) v1);
+  f2 = *((float *) v2);
+
+  if (f1 < f2)
+    return -1;
+  else if (f1 > f2)
+    return 1;
+  else
+    return 0;
+}
