@@ -14,7 +14,7 @@
   dbruce@tampabay.rr.com
 
   Part of "Tux4Kids" Project
-  http://tux4kids.alioth.debian.org/
+  http://www.tux4kids.com/
   Subversion repository:
   https://svn.debian.alioth.org/tux4kids/tuxmath/
 
@@ -30,14 +30,14 @@
 /* FIXME maybe unistd.h not needed, even less sure about portability */
 //#include <unistd.h>
 
-#include <SDL.h>
-#include <SDL_ttf.h>
+#include "SDL.h"
+#include "SDL_ttf.h"
 
 #ifndef NOSOUND
-#include <SDL_mixer.h>
+#include "SDL_mixer.h"
 #endif
 
-#include <SDL_image.h>
+#include "SDL_image.h"
 
 #include "options.h"
 #include "tuxmath.h"
@@ -59,6 +59,7 @@ SDL_Surface* images[NUM_IMAGES];
 #define NUM_FLIPPED_IMAGES 6
 SDL_Surface* flipped_images[NUM_FLIPPED_IMAGES];
 int flipped_img_lookup[NUM_IMAGES];
+SDL_Surface* blended_igloos[NUM_BLENDED_IGLOOS];
 
 const int flipped_img[] = {
   IMG_PENGUIN_WALK_ON1,
@@ -81,6 +82,7 @@ void handle_command_args(int argc, char* argv[]);
 void initialize_SDL(void);
 void load_data_files(void);
 void generate_flipped_images(void);
+void generate_blended_images(void);
 
 //int initialize_game_options(void);
 void seticon(void);
@@ -95,6 +97,7 @@ void cleanup_memory(void);
 void setup(int argc, char * argv[])
 {
   /* initialize settings and read in config files: */
+  /* Note this now only does the global settings   */
   initialize_options();
   /* Command-line code now in own function: */
   handle_command_args(argc, argv);
@@ -104,6 +107,10 @@ void setup(int argc, char * argv[])
   load_data_files();
  /* Generate flipped versions of walking images */
   generate_flipped_images();
+  /* Generate blended images (e.g., igloos) */
+  generate_blended_images();
+  /* Note that the per-user options will be set after the call to
+     titlescreen, to allow for user-login to occur. */
 }
 
 
@@ -111,8 +118,7 @@ void setup(int argc, char * argv[])
 
 /* Set up mathcards with default values for math question options, */
 /* set up game_options with defaults for general game options,     */
-/* then read in global config file, followed if desired by user's  */
-/* own config file:                                                */
+/* then read in global config file                                 */
 void initialize_options(void)
 {
   /* Initialize MathCards backend for math questions: */
@@ -139,8 +145,14 @@ void initialize_options(void)
     fprintf(stderr, "\nCould not find global config file.\n");
     /* can still proceed using hard-coded defaults.         */
   }
+}
 
-  /* Now read in user-specific settings, if desired.  By    */
+/* Read in the user-specific options (if desired)              */
+/* This has been split from the above to allow it to be called */
+/* from titlescreen, to allow for user-login to occur.         */
+void initialize_options_user(void)
+{
+  /* Read in user-specific settings, if desired.  By    */  
   /* default, this restores settings from the player's last */
   /* game:                                                  */
   if (Opts_PerUserConfig())
@@ -182,6 +194,7 @@ void initialize_options(void)
 /* Handle any arguments passed from command line */
 void handle_command_args(int argc, char* argv[])
 {
+  DIR *dirp;
   int i;
 
   for (i = 1; i < argc; i++)
@@ -206,6 +219,12 @@ void handle_command_args(int argc, char* argv[])
              "to configure the behavior of Tuxmath.\n\n");
 
       printf("Run the game with:\n"
+	"--homedir dirname      - seek for user home director(ies) in the specified\n"
+	"                         location, rather than the user's actual home\n"
+	"                         directory.  You can set up a user directory tree in\n"
+	"                         this location (see README).  This option is\n"
+	"                         especially useful for schools where all students log\n"
+	"                         in with a single user name.\n"
         "--optionfile filename  - read config settings from named file. The locations\n"
         "                         searched for a file with a matching name are the\n"
         "                         current working directory, the absolute path of the\n"
@@ -258,7 +277,25 @@ void handle_command_args(int argc, char* argv[])
 	    
       usage(0, argv[0]);
     }
-    /* TODO implement --optionfile filename */
+    else if (0 == strcmp(argv[i], "--homedir"))
+    {
+      // Parse the user choice of a non-default home directory
+      if (i >= argc -1)
+      {
+	fprintf(stderr, "%s option requires an argument (dirname)\n", argv[i]);
+	usage(1, argv[0]);
+      }
+      else // see whether the specified name is a directory
+      {
+	if ((dirp = opendir(argv[i+1])) == NULL)
+	  fprintf(stderr,"homedir: %s is not a directory, or it could not be read\n", argv[i+1]);
+	else {
+	  set_user_data_dir(argv[i+1]);  // copy the homedir setting
+	  closedir(dirp);
+	}
+	i++;   // to pass over the next argument, so remaining options parsed
+      }
+    }
     else if (0 == strcmp(argv[i], "--optionfile"))
     {
       if (i >= argc - 1)
@@ -402,6 +439,17 @@ void initialize_SDL(void)
 
   atexit(TTF_Quit); // Maybe this is redundant?
 
+#ifdef SDL_Pango
+  if (SDLPango_Init () < 0)
+  {
+      fprintf(stderr,
+            "\nWarning: I could not initialize SDL_Pango !\n"
+            "%s\n\n", SDL_GetError());
+  }
+  else {
+    init_SDLPango_Context();
+  }
+#endif
 
 
   #ifndef NOSOUND
@@ -536,6 +584,28 @@ void generate_flipped_images(void)
 }
 
 
+/* Created images that are blends of two other images to smooth out
+   the transitions. */
+void generate_blended_images(void)
+{
+  blended_igloos[0] = Blend(images[IMG_IGLOO_REBUILDING1],NULL,0.06);
+  blended_igloos[1] = Blend(images[IMG_IGLOO_REBUILDING1],NULL,0.125);
+  blended_igloos[2] = Blend(images[IMG_IGLOO_REBUILDING1],NULL,0.185);
+  blended_igloos[3] = Blend(images[IMG_IGLOO_REBUILDING1],NULL,0.25);
+  blended_igloos[4] = Blend(images[IMG_IGLOO_REBUILDING1],NULL,0.5);
+  blended_igloos[5] = Blend(images[IMG_IGLOO_REBUILDING1],NULL,0.75);
+  blended_igloos[6] = images[IMG_IGLOO_REBUILDING1];
+  blended_igloos[7] = Blend(images[IMG_IGLOO_REBUILDING2],images[IMG_IGLOO_REBUILDING1],0.25);
+  blended_igloos[8] = Blend(images[IMG_IGLOO_REBUILDING2],images[IMG_IGLOO_REBUILDING1],0.5);
+  blended_igloos[9] = Blend(images[IMG_IGLOO_REBUILDING2],images[IMG_IGLOO_REBUILDING1],0.75);
+  blended_igloos[10] = images[IMG_IGLOO_REBUILDING2];
+  blended_igloos[11] = Blend(images[IMG_IGLOO_INTACT],images[IMG_IGLOO_REBUILDING2],0.25);
+  blended_igloos[12] = Blend(images[IMG_IGLOO_INTACT],images[IMG_IGLOO_REBUILDING2],0.5);
+  blended_igloos[13] = Blend(images[IMG_IGLOO_INTACT],images[IMG_IGLOO_REBUILDING2],0.75);
+  blended_igloos[14] = images[IMG_IGLOO_INTACT];
+}
+
+
 /* save options and free heap */
 /* use for successful exit */
 void cleanup(void)
@@ -561,6 +631,11 @@ void cleanup_on_error(void)
 
 /* free any heap memory used during game DSB */
 /* and also quit SDL properly:               */
+/* NOTE - this function will get called twice if   */
+/* exit occurs because of window close, so we      */
+/* need to check all pointers before freeing them, */
+/* and set them to NULL after freeing them, so we  */
+/* avoid segfaults at exit from double free()      */
 void cleanup_memory(void)
 {
   /* Free all images and sounds used by SDL: */
@@ -568,8 +643,12 @@ void cleanup_memory(void)
   int frequency,channels,n_timesopened;
   Uint16 format;
 
-  TTF_CloseFont(default_font);
-  TTF_Quit();
+  if(default_font)
+  {
+    TTF_CloseFont(default_font);
+    default_font = NULL;
+    TTF_Quit();
+  }
 
   for (i = 0; i < NUM_IMAGES; i++)
   {
@@ -592,16 +671,39 @@ void cleanup_memory(void)
     musics[i] = NULL;
   }
 
-  for (i = 0; i < num_lessons; i++) {
-    free(lesson_list_titles[i]);
-    free(lesson_list_filenames[i]);
+  if (lesson_list_titles)
+  {
+    for (i = 0; i < num_lessons; i++)
+    {
+      if (lesson_list_titles[i])
+      {
+        free(lesson_list_titles[i]);
+        lesson_list_titles[i] = NULL;
+      }
+    }
+    free(lesson_list_titles);
+    lesson_list_titles = NULL;
+  }
+
+  if (lesson_list_filenames)
+  {
+    for (i = 0; i < num_lessons; i++)
+    {
+      if (lesson_list_filenames[i])
+      {
+        free(lesson_list_filenames[i]);
+        lesson_list_filenames[i] = NULL;
+      }
+    }
+    free(lesson_list_filenames);
+    lesson_list_filenames = NULL;
+  }
+
+  if (lesson_list_goldstars)
+  {
+    free(lesson_list_goldstars);
+    lesson_list_goldstars = NULL;
   }
-  free(lesson_list_titles);
-  free(lesson_list_filenames);
-  free(lesson_list_goldstars);
-  lesson_list_titles = NULL;
-  lesson_list_filenames = NULL;
-  lesson_list_goldstars = NULL;
 
   // Close the audio mixer. We have to do this at least as many times
   // as it was opened.
@@ -610,6 +712,11 @@ void cleanup_memory(void)
     Mix_CloseAudio();
     n_timesopened--;
   }
+  
+#ifdef SDL_Pango
+   free_SDLPango_Context();
+#endif
+
 
   // Finally, quit SDL
   SDL_Quit();
@@ -622,10 +729,6 @@ void cleanup_memory(void)
 
 
 
-
-
-
-
 /* Set the application's icon: */
 
 void seticon(void)
