@@ -24,39 +24,27 @@
 
 #include "config.h"
 
-// Translation stuff:
-/* NLS not active on Mac yet: */
-#ifdef MACOSX
-#define N_(String) String
-#define _(String) String
-#else
-/* Nor on Windows: */
-#ifdef WIN32
-#define N_(String) String
-#define _(String) String
-/* Set up support for NLS on other platforms - i.e. Linux/Unix: */
-#else
+// Translation stuff (now works for Mac and Win too!): 
 #include "gettext.h"
 #include <locale.h>
 #define _(String) gettext (String)
 #define gettext_noop(String) String
 #define N_(String) gettext_noop (String)
-#endif
-#endif
 
 #include <wchar.h>
 
 #include "SDL.h"
 #include "SDL_ttf.h"
-#include "config.h"
 
 #ifndef NOSOUND
 #include "SDL_mixer.h"
 #endif
 
 //#define NOSOUND
-//#define TUXMATH_DEBUG   /* for conditional compilation of debugging output */
-//#define FEEDBACK_DEBUG  /* for Tim's feedback speed control code           */
+/* for conditional compilation of debugging output */
+//#define TUXMATH_DEBUG
+/* for Tim's feedback speed control code           */
+//#define FEEDBACK_DEBUG
 
 /* Maximum length of file path: */
 #define PATH_MAX 4096
@@ -103,9 +91,9 @@
 #define DEFAULT_NUM_BKGDS 6
 #define DEFAULT_MAX_CITY_COLORS 4
 
-#define MINIMUM_SPEED 0.3
+#define MINIMUM_SPEED 0.8
 #define MAX_MAX_SPEED 20.0
-#define MIN_SPEEDUP_FACTOR 0.5
+#define MIN_SPEEDUP_FACTOR 1
 #define MAX_SPEEDUP_FACTOR 2.0
 #define MAX_BONUS_SPEED_RATIO 3.0
 #define MIN_COMETS 1
@@ -147,6 +135,8 @@ extern SDL_Color yellow;
 extern SDL_Surface* screen; /* declared in setup.c; also used in game.c, options.c, fileops.c, credits.c, titlescreen.c */
 extern SDL_Surface* images[];    /* declared in setup.c, used in same files as screen */
 extern SDL_Surface* flipped_images[];
+#define NUM_BLENDED_IGLOOS 15
+extern SDL_Surface* blended_igloos[];
 extern int flipped_img_lookup[];
 
 extern TTF_Font  *default_font;
